-- Samizdat Database Creation - PostgreSQL
--
--   Copyright (c) 2002-2006  Dmitry Borodaenko <angdraug@debian.org>
--
--   This program is free software.
--   You can distribute/modify this program under the terms of
--   the GNU General Public License version 2 or later.
--

-- RDF Data Model
CREATE TABLE Resource (
	id SERIAL PRIMARY KEY,
	published_date TIMESTAMP WITH TIME ZONE -- received date with site tz
		DEFAULT CURRENT_TIMESTAMP NOT NULL,
	literal BOOLEAN DEFAULT false,
	uriref BOOLEAN DEFAULT false,
	label TEXT); -- literal value | external uriref | internal class name
	-- optimize: store external uriref hash in numeric field

CREATE INDEX Resource_label_idx ON Resource (label);

CREATE TABLE Statement (
	id INTEGER PRIMARY KEY REFERENCES Resource,
	subject INTEGER NOT NULL REFERENCES Resource,
	predicate INTEGER NOT NULL REFERENCES Resource,
	object INTEGER NOT NULL REFERENCES Resource,
	rating NUMERIC(4,2)); -- computed from Vote

CREATE INDEX Statement_subject_idx ON Statement (subject);

-- Members and Sessions
CREATE TABLE Member (
	id INTEGER PRIMARY KEY REFERENCES Resource,
	login TEXT UNIQUE NOT NULL,
	full_name TEXT NOT NULL,
	email TEXT UNIQUE NOT NULL,
	passwd TEXT,
	prefs TEXT,
	confirm TEXT UNIQUE,
	session TEXT UNIQUE,
	login_time TIMESTAMP WITH TIME ZONE,
	last_time TIMESTAMP WITH TIME ZONE);

-- Messages and Threads
CREATE TABLE Message (
	id INTEGER PRIMARY KEY REFERENCES Resource,
	parent INTEGER REFERENCES Message,   -- In-Reply-To:
	description INTEGER REFERENCES Message,   -- abstract or toc or thumbnail
	version_of INTEGER REFERENCES Message,   -- current version
	open BOOLEAN DEFAULT false,   -- editing open for all members
	hidden BOOLEAN DEFAULT false,   -- hidden from public view
	creator INTEGER REFERENCES Member,   -- From:
	language TEXT,   -- language code
	title TEXT NOT NULL,   -- Subject:
	format TEXT,   -- MIME type
	content TEXT,
	html_full TEXT,
	html_short TEXT);

CREATE INDEX Message_parent_idx ON Message (parent);

CREATE INDEX Message_version_of_idx ON Message (version_of);

-- Voting Data
CREATE TABLE Vote (
	id INTEGER PRIMARY KEY REFERENCES Resource,
	proposition INTEGER REFERENCES Statement,
	member INTEGER REFERENCES Member,
	rating NUMERIC(2),
	UNIQUE (proposition, member));

-- Moderation Actions Log
CREATE TABLE Moderation (
	action_date TIMESTAMP WITH TIME ZONE
		DEFAULT CURRENT_TIMESTAMP PRIMARY KEY,
	moderator INTEGER REFERENCES Member,
	action TEXT,
	resource INTEGER REFERENCES Resource);

-- Material Items Sharing
CREATE TABLE Item (
	id INTEGER PRIMARY KEY REFERENCES Resource,
	description INTEGER REFERENCES Message,
	contributor INTEGER REFERENCES Member,
	possessor INTEGER REFERENCES Member);

CREATE TABLE Possession (
	id INTEGER PRIMARY KEY REFERENCES Resource,
	item INTEGER REFERENCES Item,
	taken_from INTEGER REFERENCES Member,
	given_to INTEGER REFERENCES Member);
